# Manual page qdrouterd.8
<!-- -*- markdown -*-
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License
-->

## Name

qdrouterd - AMQP message router.

## Synopsis

qdrouterd [*options*]

## Description

The Qpid Dispatch router (qdrouterd) is a network daemon that directs
AMQP 1.0 messages between endpoints, such as messaging clients and
servers.
## Options

-c, --config=PATH (/etc/qpid-dispatch/qdrouterd.conf)
:   Load configuration from file at PATH

-I, --include=PATH (/usr/lib/qpid-dispatch/python)
:   Location of Dispatch's Python library

-d, --daemon
:   Run process as a SysV-style daemon

-P, --pidfile
:   If daemon, the file for the stored daemon pid

-U, --user
:   If daemon, the username to run as

-h, --help
:   Print this help



Run `qdrouterd --help` to see options.

## Files

/etc/qpid-dispatch/qdrouterd.conf
:   Configuration flie.

## See Also
*qdrouterd.conf*(5), *qdstat*(8), *qdstat.conf*(5),

[http://qpid.apache.org/components/dispatch-router](http://qpid.apache.org/components/dispatch-router)


# Manual page qdstat.8
<!--*- markdown -*-
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License
-->

## Name

qdstat - A tool to inspect Dispatch router networks

## Synopsis

qdrouterd [*options*]

## Description

*qdstat* shows status information about networks of Dispatch routers.
It can display connections, network nodes and links, and router stats
such as memory use.
## Options

-h, --help
:   show this help message and exit

--version
:   Print version and exit.

-g, --general
:   Show General Router Stats

-c, --connections
:   Show Connections

-l, --links
:   Show Router Links

-n, --nodes
:   Show Router Nodes

-a, --address
:   Show Router Addresses

-m, --memory
:   Show Broker Memory Stats

### Connection Options

-b URL, --bus=URL
:   URL of the messaging bus to connect to (default

-r ROUTER-ID, --router=ROUTER-ID
:   Router to be queried

-t SECS, --timeout=SECS
:   Maximum time to wait for connection in seconds (default 5)

--sasl-mechanism=MECH
:   Force SASL mechanism (e.g. EXTERNAL, ANONYMOUS, PLAIN, CRAM-MD5, DIGEST-MD5, GSSAPI).

--ssl-certificate=CERT
:   Client SSL certificate (PEM Format)

--ssl-key=KEY
:   Client SSL private key (PEM Format)



Run `qdstat --help` to see options.

## See Also

*qdrouterd* (8), *qdmanage* (8), *qdrouterd.conf* (5)

[http://qpid.apache.org/components/dispatch-router](http://qpid.apache.org/components/dispatch-router)

# Manual page qdmanage.8
## Name

qdmanage - A management tool for Dispatch routers.

## Synopsis

qdmanage *operation* [*options...*] [*arguments...*]

## Description

An AMQP management client for use with qdrouterd. Sends AMQP management
operations requests and prints the response in JSON format.  This is a generic AMQP
management tool and can be used with any standard AMQP managed endpoint, not just
with qdrouter.

## Operations

`query` [*ATTR*...]
:   Prints the named attributes of all entities. With no arguments prints all attributes.
    The --type option restricts the result to entities extending the type.

`create` [*ATTR=VALUE*...]
:   Create a new entity with the specified attributes.
    With the --stdin option, read attributes from stdin. This can be a JSON
    map of attributes to create a single entity, or a JSON list of maps to create multiple entitiees.

`read`
:   Print the attributes of an entity specified by the --name or --identity options.
    With the --stdin option, create entities based on data from stdin. This can be a JSON
    map of attributes to create a single entity, or a JSON list of maps to create multiple entitiees.

`update` [*ATTR=VALUE*...]
:   Update the attributes of an existing entity.
    With the --stdin option, read attributes from stdin. This can be a JSON
    map of attributes to update a single entity, or a JSON list of maps to update multiple entitiees.
    If an ATTR name is listed with no =VALUE, that attribute will be deleted from the entity.

`delete`
:    Delete an entity specified by the --name or --identity options.

`get-types` [*TYPE*]
:    List entity types with their base types. With no arguments list all types.

`get-operations` [*TYPE*]
:    List entity types with their operations. With no arguments list all types.

`get-attributes` [*TYPE*]
:    List entity types with their attributes.  With no arguments list all types.

`get-annotations` [*TYPE*]
:    List entity types with their annotations.  With no arguments list all types.

`get-mgmt-nodes`
:    List all other known management nodes connected to this one.
## Options

-h, --help
:   show this help message and exit

--version
:   Print version and exit.

--type=TYPE
:   Type of entity to operate on.

--name=NAME
:   Name of entity to operate on.

--identity=ID
:   Identity of entity to operate on.

--indent=INDENT
:   Pretty-printing indent. -1 means don't pretty-print

--stdin
:   Read attributes as JSON map or list of maps from

### Connection Options

-b URL, --bus=URL
:   URL of the messaging bus to connect to (default

-r ROUTER-ID, --router=ROUTER-ID
:   Router to be queried

-t SECS, --timeout=SECS
:   Maximum time to wait for connection in seconds (default 5)

--sasl-mechanism=MECH
:   Force SASL mechanism (e.g. EXTERNAL, ANONYMOUS, PLAIN, CRAM-MD5, DIGEST-MD5, GSSAPI).

--ssl-certificate=CERT
:   Client SSL certificate (PEM Format)

--ssl-key=KEY
:   Client SSL private key (PEM Format)



Run `qdmanage --help` to see options.

## Files

*/usr//usr/share/doc/qpid-dispatch/qdrouter.json*
:    Management schema for qdrouterd.
*/usr//usr/share/doc/qpid-dispatch/qdrouter.json.readme.txt*
:    Explanation of the management schema.

## Examples

Show the logging configuration

    qdmanage query --type=log

Enable debug and higher log messages by default:

    qdmanage udpdate name=log/DEFAULT enable=debug+

Enable trace log messages only for the MESSAGE moudle and direct MESSAGE logs to the file "test.log"

    qdmanage udpdate name=log/MESSAGE enable=trace output=test.log

Set MESSAGE logging back to the default:

    qdmanage udpdate name=log/MESSAGE enable=default

Disable MESSAGE logging:

    qdmanage udpdate name=log/MESSAGE enable=none

## See Also

*qdrouterd*(8), *qdstat*(8), *qdrouterd.conf*(5)

[http://qpid.apache.org/components/dispatch-router](http://qpid.apache.org/components/dispatch-router)




# Manual page qdrouterd.conf.5

## Name

qdrouterd.conf - Configuration file for the Qpid Dispatch router

## Description

The configuration file is made up of sections with this syntax:

    SECTION-NAME {
        ATTRIBUTE-NAME: ATTRIBUTE-VALUE
        ATTRIBUTE-NAME: ATTRIBUTE-VALUE
        ...
    }

There are two types of sections:

*Configuration sections* correspond to configuration entities. They can be queried and
configured via management tools as well as via the configuration file.

*Annotation sections* define a group of attribute values that can be included in
one or more entity sections.

For example you can define an "ssl-profile" annotation section with SSL credentials
that can be included in multiple "listener" entities. Here's an example, note
how the 'ssl-profile' attribute of 'listener' sections references the 'name'
attribute of 'ssl-profile' sections.

    ssl-profile {
        name: ssl-profile-one
        cert-db: ca-certificate-1.pem
        cert-file: server-certificate-1.pem
        key-file: server-private-key.pem
    }

    listener {
        ssl-profile: ssl-profile-one
        addr: 0.0.0.0
        port: 20102
        sasl-mechanisms: ANONYMOUS
    }


## Annotation Sections



### Addrport

Attributes for internet address and port.

addr (String, default=0.0.0.0)
:   Host address: ipv4 or ipv6 literal or a host name.

port (String, default=amqp)
:   Port number or symbolic service name.


Used by listener, connector.


### Saslmechanisms

Attribute for a list of SASL mechanisms.

saslMechanisms (String, required)
:   Comma separated list of accepted SASL authentication mechanisms.


Used by listener, connector.


### Connectionrole

Attribute for the role of a connection.

role (One of [normal, inter-router, on-demand], default=normal)
:   The role of an established connection. In the normal role, the connection is assumed to be used for AMQP clients that are doing normal message delivery over the connection.  In the inter-router role, the connection is assumed to be to another router in the network.  Inter-router discovery and routing protocols can only be used over interRouter connections.


Used by listener, connector.


### Sslprofile

Attributes for setting TLS/SSL configuration for connections.

certDb (String)
:   The path to the database that contains the public certificates of trusted certificate authorities (CAs).

certFile (String)
:   The path to the file containing the PEM-formatted public certificate to be used on the local end of any connections using this profile.

keyFile (String)
:   The path to the file containing the PEM-formatted private key for the above certificate.

passwordFile (String)
:   If the above private key is password protected, this is the path to a file containing the password that unlocks the certificate key.

password (String)
:   An alternative to storing the password in a file referenced by passwordFile is to supply the password right here in the configuration file.  This option can be used by supplying the password in the 'password' option.  Don't use both password and passwordFile in the same profile.


Used by listener, connector.


## Configuration Sections


### Container

Attributes related to the AMQP container.

containerName (String)
:   The  name of the AMQP container.  If not specified, the container name will be set to a value of the container's choosing.  The automatically assigned container name is not guaranteed to be persistent across restarts of the container.

workerThreads (Integer, default=1)
:   The number of threads that will be created to process message traffic and other application work (timers, non-amqp file descriptors, etc.) .

debugDump (String)
:   A file to dump debugging information that can't be logged normally.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



### Router

Tracks peer routers and computes routes to destinations.

routerId (String)
:   Router's unique identity.

mode (One of [standalone, interior, edge, endpoint], default=standalone)
:   In standalone mode, the router operates as a single component.  It does not participate in the routing protocol and therefore will not coorperate with other routers. In interior mode, the router operates in cooreration with other interior routers in an interconnected network.  In edge mode, the router operates with an uplink into an interior router network. Edge routers are typically used as connection concentrators or as security firewalls for access into the interior network.

area (String)
:   Unused placeholder.

helloInterval (Integer, default=1)
:   Interval in seconds between HELLO messages sent to neighbor routers.

helloMaxAge (Integer, default=3)
:   Time in seconds after which a neighbor is declared lost if no HELLO is received.

raInterval (Integer, default=30)
:   Interval in seconds between Router-Advertisements sent to all routers.

remoteLsMaxAge (Integer, default=60)
:   Time in seconds after which link state is declared stale if no RA is received.

mobileAddrMaxAge (Integer, default=60)
:   Time in seconds after which mobile addresses are declared stale if no RA is received.

addrCount (Integer)
:   Number of addresses known to the router.

linkCount (Integer)
:   Number of links attached to the router node.

nodeCount (Integer)
:   Number of known peer router nodes.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



### Listener

Listens for incoming connections to the router.

requirePeerAuth (Boolean, default=True)
:   Only for listeners using SSL.  If set to 'yes', attached clients will be required to supply a certificate.  If the certificate is not traceable to a CA in the ssl profile's cert-db, authentication fails for the connection.

trustedCerts (String)
:   This optional setting can be used to reduce the set of available CAs for client authentication.  If used, this setting must provide a path to a PEM file that contains the trusted certificates.

allowUnsecured (Boolean)
:   For listeners using SSL only.  If set to 'yes', this option causes the listener to watch the initial network traffic to determine if the client is using SSL or is running in-the-clear.  The listener will enable SSL only if the client uis using SSL.

allowNoSasl (Boolean)
:   If set to 'yes', this option causes the listener to allow clients to connect even if they skip the SASL authentication protocol.

maxFrameSize (Integer, default=65536)
:   Defaults to 65536.  If specified, it is the maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.


Annotations sslProfile, addrPort, saslMechanisms, connectionRole.

### Connector

Establishes an outgoing connections from the router.

allowRedirect (Boolean, default=True)
:   Allow the peer to redirect this connection to another address.

maxFrameSize (Integer, default=65536)
:   Maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.


Annotations sslProfile, addrPort, saslMechanisms, connectionRole.

### Log

Configure logging for a particular module.

module (One of [ROUTER, MESSAGE, SERVER, AGENT, CONTAINER, CONFIG, ERROR, DISPATCH, DEFAULT], required)
:   Module to configure. The special module 'DEFAULT' specifies defaults for all modules.

enable (String, required, default=default)
:   Levels are: trace, debug, info, notice, warning, error, critical. The enable string is a comma-separated list of levels. A level may have a trailing '+' to enable that level and above. For example 'trace,debug,warning+' means enable trace, debug, warning, error and critical. The value 'none' means disable logging for the module. The value 'default' means use the value from the DEFAULT module.

timestamp (Boolean)
:   Include timestamp in log messages.

source (Boolean)
:   Include source file and line number in log messages.

output (String)
:   Where to send log messages. Can be 'stderr', 'syslog' or a file name.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



### Fixedaddress

Establishes semantics for addresses starting with a prefix.

prefix (String, required)
:   The address prefix (always starting with '/').

phase (Integer)
:   The phase of a multi-hop address passing through one or more waypoints.

fanout (One of [multiple, single], default=multiple)
:   One of 'multiple' or 'single'.  Multiple fanout is a non-competing pattern.  If there are multiple consumers using the same address, each consumer will receive its own copy of every message sent to the address.  Single fanout is a competing pattern where each message is sent to only one consumer.

bias (One of [closest, spread], default=closest)
:   Only if fanout is single.  One of 'closest' or 'spread'.  Closest bias means that messages to an address will always be delivered to the closest (lowest cost) subscribed consumer. Spread bias will distribute the messages across subscribers in an approximately even manner.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



### Waypoint

A remote node that messages for an address pass through.

address (String, required)
:   The AMQP address of the waypoint.

connector (String, required)
:   The name of the on-demand connector used to reach the waypoint's container.

inPhase (Integer, default=-1)
:   The phase of the address as it is routed _to_ the waypoint.

outPhase (Integer, default=-1)
:   The phase of the address as it is routed _from_ the waypoint.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



