
# Name

qdrouterd.conf - Configuration file for the Qpid Dispatch router

# Description

The configuration file is made up of sections with this syntax:

    SECTION-NAME {
        ATTRIBUTE-NAME: ATTRIBUTE-VALUE
        ATTRIBUTE-NAME: ATTRIBUTE-VALUE
        ...
    }

There are two types of sections:

*Configuration sections* correspond to configuration entities. They can be queried and
configured via management tools as well as via the configuration file.

*Annotation sections* define a group of attribute values that can be included in
one or more entity sections.

For example you can define an "ssl-profile" annotation section with SSL credentials
that can be included in multiple "listener" entities. Here's an example, note
how the 'ssl-profile' attribute of 'listener' sections references the 'name'
attribute of 'ssl-profile' sections.

    ssl-profile {
        name: ssl-profile-one
        cert-db: ca-certificate-1.pem
        cert-file: server-certificate-1.pem
        key-file: server-private-key.pem
    }

    listener {
        ssl-profile: ssl-profile-one
        addr: 0.0.0.0
        port: 20102
        sasl-mechanisms: ANONYMOUS
    }


# Annotation Sections



## addrPort

Attributes for internet address and port.

addr (String, default=0.0.0.0)
:   Host address: ipv4 or ipv6 literal or a host name.

port (String, default=amqp)
:   Port number or symbolic service name.


Used by listener, connector.


## saslMechanisms

Attribute for a list of SASL mechanisms.

saslMechanisms (String, required)
:   Comma separated list of accepted SASL authentication mechanisms.


Used by listener, connector.


## connectionRole

Attribute for the role of a connection.

role (One of [normal, inter-router, on-demand], default=normal)
:   The role of an established connection. In the normal role, the connection is assumed to be used for AMQP clients that are doing normal message delivery over the connection.  In the inter-router role, the connection is assumed to be to another router in the network.  Inter-router discovery and routing protocols can only be used over interRouter connections.


Used by listener, connector.


## sslProfile

Attributes for setting TLS/SSL configuration for connections.

certDb (String)
:   The path to the database that contains the public certificates of trusted certificate authorities (CAs).

certFile (String)
:   The path to the file containing the PEM-formatted public certificate to be used on the local end of any connections using this profile.

keyFile (String)
:   The path to the file containing the PEM-formatted private key for the above certificate.

passwordFile (String)
:   If the above private key is password protected, this is the path to a file containing the password that unlocks the certificate key.

password (String)
:   An alternative to storing the password in a file referenced by passwordFile is to supply the password right here in the configuration file.  This option can be used by supplying the password in the 'password' option.  Don't use both password and passwordFile in the same profile.


Used by listener, connector.


# Configuration Sections


## container

Attributes related to the AMQP container.

containerName (String)
:   The  name of the AMQP container.  If not specified, the container name will be set to a value of the container's choosing.  The automatically assigned container name is not guaranteed to be persistent across restarts of the container.

workerThreads (Integer, default=1)
:   The number of threads that will be created to process message traffic and other application work (timers, non-amqp file descriptors, etc.) .

debugDump (String)
:   A file to dump debugging information that can't be logged normally.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



## router

Tracks peer routers and computes routes to destinations.

routerId (String)
:   Router's unique identity.

mode (One of [standalone, interior, edge, endpoint], default=standalone)
:   In standalone mode, the router operates as a single component.  It does not participate in the routing protocol and therefore will not coorperate with other routers. In interior mode, the router operates in cooreration with other interior routers in an interconnected network.  In edge mode, the router operates with an uplink into an interior router network. Edge routers are typically used as connection concentrators or as security firewalls for access into the interior network.

area (String)
:   Unused placeholder.

helloInterval (Integer, default=1)
:   Interval in seconds between HELLO messages sent to neighbor routers.

helloMaxAge (Integer, default=3)
:   Time in seconds after which a neighbor is declared lost if no HELLO is received.

raInterval (Integer, default=30)
:   Interval in seconds between Router-Advertisements sent to all routers.

remoteLsMaxAge (Integer, default=60)
:   Time in seconds after which link state is declared stale if no RA is received.

mobileAddrMaxAge (Integer, default=60)
:   Time in seconds after which mobile addresses are declared stale if no RA is received.

addrCount (Integer)
:   Number of addresses known to the router.

linkCount (Integer)
:   Number of links attached to the router node.

nodeCount (Integer)
:   Number of known peer router nodes.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



## listener

Listens for incoming connections to the router.

requirePeerAuth (Boolean, default=True)
:   Only for listeners using SSL.  If set to 'yes', attached clients will be required to supply a certificate.  If the certificate is not traceable to a CA in the ssl profile's cert-db, authentication fails for the connection.

trustedCerts (String)
:   This optional setting can be used to reduce the set of available CAs for client authentication.  If used, this setting must provide a path to a PEM file that contains the trusted certificates.

allowUnsecured (Boolean)
:   For listeners using SSL only.  If set to 'yes', this option causes the listener to watch the initial network traffic to determine if the client is using SSL or is running in-the-clear.  The listener will enable SSL only if the client uis using SSL.

allowNoSasl (Boolean)
:   If set to 'yes', this option causes the listener to allow clients to connect even if they skip the SASL authentication protocol.

maxFrameSize (Integer, default=65536)
:   Defaults to 65536.  If specified, it is the maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.


Annotations sslProfile, addrPort, saslMechanisms, connectionRole.

## connector

Establishes an outgoing connections from the router.

allowRedirect (Boolean, default=True)
:   Allow the peer to redirect this connection to another address.

maxFrameSize (Integer, default=65536)
:   Maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.


Annotations sslProfile, addrPort, saslMechanisms, connectionRole.

## log

Configure logging for a particular module.

module (One of [ROUTER, MESSAGE, SERVER, AGENT, CONTAINER, CONFIG, ERROR, DISPATCH, DEFAULT], required)
:   Module to configure. The special module 'DEFAULT' specifies defaults for all modules.

enable (String, required, default=default)
:   Levels are: trace, debug, info, notice, warning, error, critical. The enable string is a comma-separated list of levels. A level may have a trailing '+' to enable that level and above. For example 'trace,debug,warning+' means enable trace, debug, warning, error and critical. The value 'none' means disable logging for the module. The value 'default' means use the value from the DEFAULT module.

timestamp (Boolean)
:   Include timestamp in log messages.

source (Boolean)
:   Include source file and line number in log messages.

output (String)
:   Where to send log messages. Can be 'stderr', 'syslog' or a file name.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



## fixedAddress

Establishes semantics for addresses starting with a prefix.

prefix (String, required)
:   The address prefix (always starting with '/').

phase (Integer)
:   The phase of a multi-hop address passing through one or more waypoints.

fanout (One of [multiple, single], default=multiple)
:   One of 'multiple' or 'single'.  Multiple fanout is a non-competing pattern.  If there are multiple consumers using the same address, each consumer will receive its own copy of every message sent to the address.  Single fanout is a competing pattern where each message is sent to only one consumer.

bias (One of [closest, spread], default=closest)
:   Only if fanout is single.  One of 'closest' or 'spread'.  Closest bias means that messages to an address will always be delivered to the closest (lowest cost) subscribed consumer. Spread bias will distribute the messages across subscribers in an approximately even manner.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.



## waypoint

A remote node that messages for an address pass through.

address (String, required)
:   The AMQP address of the waypoint.

connector (String, required)
:   The name of the on-demand connector used to reach the waypoint's container.

inPhase (Integer, default=-1)
:   The phase of the address as it is routed _to_ the waypoint.

outPhase (Integer, default=-1)
:   The phase of the address as it is routed _from_ the waypoint.

name (String, unique)
:   Unique name, can be changed.

identity (String, unique)
:   Unique identity, will not change.


